/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: LogViewer.java,v 1.58 2005/04/26 15:40:58 ewchan Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.log.ui.internal.views;


import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.hyades.log.ui.internal.LogUIPlugin;
import org.eclipse.hyades.log.ui.internal.LogUIPluginImages;
import org.eclipse.hyades.log.ui.internal.actions.provider.ILogFilterProvider;
import org.eclipse.hyades.log.ui.internal.actions.provider.ILogFindProvider;
import org.eclipse.hyades.log.ui.internal.util.CbeEmfUtility;
import org.eclipse.hyades.log.ui.internal.util.ColumnsDialog;
import org.eclipse.hyades.log.ui.internal.util.FeatureNode;
import org.eclipse.hyades.log.ui.internal.util.FilterTableElement;
import org.eclipse.hyades.log.ui.internal.util.FiltersDialog;
import org.eclipse.hyades.log.ui.internal.util.LogFindCriteria;
import org.eclipse.hyades.log.ui.internal.util.LogRecordSearch;
import org.eclipse.hyades.log.ui.internal.util.LogUIConstants;
import org.eclipse.hyades.log.ui.internal.util.LogUtil;
import org.eclipse.hyades.log.ui.internal.util.RecordChangeEvent;
import org.eclipse.hyades.log.ui.internal.util.SearchRecordDialog;
import org.eclipse.hyades.log.ui.internal.util.SortDialog;
import org.eclipse.hyades.log.ui.internal.util.TerminalNode;
import org.eclipse.hyades.log.ui.internal.util.TreeNode;
import org.eclipse.hyades.logging.events.cbe.AssociationEngine;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.util.EventFormatter;
import org.eclipse.hyades.models.cbe.CBECommonBaseEvent;
import org.eclipse.hyades.models.cbe.impl.CBEPackageImpl;
import org.eclipse.hyades.models.cbe.util.LogQueryBuilder;
import org.eclipse.hyades.models.hierarchy.CorrelationContainerProxy;
import org.eclipse.hyades.models.hierarchy.CorrelationEngine;
import org.eclipse.hyades.models.hierarchy.HierarchyPackage;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCMonitor;
import org.eclipse.hyades.models.hierarchy.TRCNode;
import org.eclipse.hyades.models.hierarchy.TRCProcessProxy;
import org.eclipse.hyades.models.hierarchy.extensions.ExtensionsFactory;
import org.eclipse.hyades.models.hierarchy.extensions.QueryResult;
import org.eclipse.hyades.models.hierarchy.extensions.ResultEntry;
import org.eclipse.hyades.models.hierarchy.extensions.SimpleOperand;
import org.eclipse.hyades.models.hierarchy.extensions.SimpleSearchQuery;
import org.eclipse.hyades.models.hierarchy.util.ILogFilterCriteria;
import org.eclipse.hyades.models.hierarchy.util.internal.HierarchyXMIQueryEngine;
import org.eclipse.hyades.models.hierarchy.util.internal.QueryUtils;
import org.eclipse.hyades.trace.ui.HyadesConstants;
import org.eclipse.hyades.trace.ui.HyadesUtil;
import org.eclipse.hyades.trace.ui.ProfileEvent;
import org.eclipse.hyades.trace.ui.TraceViewer;
import org.eclipse.hyades.trace.ui.TraceViewerPage;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.ui.internal.filters.FilterTraceViewer;
import org.eclipse.hyades.ui.internal.navigator.INavigator;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.part.WorkbenchPart;

public class LogViewer extends FilterTraceViewer implements ILogFindProvider, ILogFilterProvider{
	protected final String _title =
		LogUIPlugin.getResourceString("STR_WAS_VIEW_TITLE");
	private LogPage logPage;
	private Action pageDown;
	private Action pageUp;
	private Action goToPage;
	private Action exportRecords;
	
	private LogRecordSearch logRecordSearch = null;
	private Object lastMatch = null;
	private List cbeList;
	private SearchRecordDialog searchDialog = null;
	private Hashtable associationMap;
	private EObject prevEObject = null;
	
	private boolean handleRefreshEvent =false;
	private boolean filterRefreshEvent =false;	
	
	
	public LogViewer() {
		super();
	}
	/**
	 * Creates a log viewer page
	 * Creation date: (08/01/2000 4:17:37 PM)
	 * @return TraceViewerPage
	 */
	public TraceViewerPage createPage(EObject mofObject) {
		logPage = new LogPage(mofObject, this);
		return logPage;
	}

	public void handleProfileEvent(ProfileEvent event) {

		if (event.getType() == ProfileEvent.REFRESH_VIEWS
			|| event.getType() == ProfileEvent.UPDATE_DELTA_TIME)			
			{
				
			if(!fPartVisible)
			{		
				//ignore the events and keep track that the page need to be updated			
			   fRefreshView = true;		   
				return;
			}
			refresh();
		}
	}

	protected void refresh(){
		handleRefreshEvent=true;
		//refreshPage(getObjectToView((EObject) event.getSource()));
		if(logPage!=null && logPage.getView()!=null && logPage.getView().getViewer()!=null)
			logPage.getView().getViewer().refresh();
		handleRefreshEvent=false;		
	}
	/**	 
	 * Disposes every cached page and 
	 * cleans up the hashtable that caches the log (trace) viewer pages
	 * @see TraceViewer#dispose
	 * @see IWorkbenchPart#dispose
	 */
	public void dispose() {
		super.dispose();
	}
	/**
	 * Creates actions for opening the filter and sort dialogs
	 * and adds them to the toolbar
	 * Creation date: (1/11/2001 2:54:24 PM)
	 */
	public void makeActions() {
		if (isInitializedMenu()) {
			return; //create the toolbar only once
		}

		super.makeActions();
		
		initializedMenu(true);

		String STR_SORT = LogUIPlugin.getResourceBundle().getString("STR_SORT");
		Action sort = new Action(STR_SORT) {
			public void run() {
				SortDialog dialog =
					new SortDialog(
						getViewSite().getShell(),
						LogUIPlugin.getResourceString(
							"STR_ST_SORT_COL_SETTINGS"),
						null);
				dialog.open();
				if (dialog.getReturnCode() == Window.OK) {
					final IPage page = getCurrentPage();

					if (page != null && page instanceof LogPage) {

						org
							.eclipse
							.swt
							.custom
							.BusyIndicator
							.showWhile(null, new Runnable() {
							public void run() {

								((LogPage) page).getView().update();
							}
						});
					}
				}

			}
		};
		sort.setText(STR_SORT);
		LogUIPluginImages.setImageDescriptors(
			sort,
			LogUIPluginImages.T_LCL,
			LogUIPluginImages.IMG_SORT);
		sort.setDescription(STR_SORT);
		sort.setToolTipText(STR_SORT);

		String STR_FILTER =
			LogUIPlugin.getResourceBundle().getString("STR_FILTER");
		Action filter = new Action(STR_FILTER) {
			public void run() {
				ColumnsDialog dialog =
					new ColumnsDialog(
						getViewSite().getShell(),
						LogUIPlugin.getResourceString("STR_ST_COL_SETTINGS"),
						null);
				dialog.open();
			}
		};
		filter.setText(STR_FILTER);
		LogUIPluginImages.setImageDescriptors(
			filter,
			LogUIPluginImages.T_LCL,
			LogUIPluginImages.IMG_FILTER);
		filter.setDescription(STR_FILTER);
		filter.setToolTipText(STR_FILTER);

//		String STR_ADV_FILTER =
//			LogUIPlugin.getResourceBundle().getString("STR_ADV_FILTER");
//		Action advfilter = new Action(STR_ADV_FILTER) {
//			public void run() {
//					filter();
//			}
//		};
//		advfilter.setText(STR_ADV_FILTER);
//		LogUIPluginImages.setImageDescriptors(
//			advfilter,
//			LogUIPluginImages.T_LCL,
//			LogUIPluginImages.IMG_ADV_FILTER);
//		advfilter.setDescription(STR_ADV_FILTER);
//		advfilter.setToolTipText(STR_ADV_FILTER);

		String strPrintColumns = LogUIPlugin.getResourceBundle().getString("STR_EXPORT_RECORD");
		exportRecords = new Action(strPrintColumns) {
			public void run() {
					exportRecords();
			}
		};
		exportRecords.setText(strPrintColumns);
		LogUIPluginImages.setImageDescriptors(
			exportRecords,
			LogUIPluginImages.T_LCL,
			LogUIPluginImages.IMG_EXPORT_TO_XML);
		exportRecords.setDescription(strPrintColumns);
		exportRecords.setToolTipText(strPrintColumns);
						
		Action searchRecord = createSearchButton();
		
		getViewSite().getActionBars().setGlobalActionHandler(ActionFactory.FIND.getId(),searchRecord);

		createPageUpButton();
		createPageDownButton();
		createGoToPageButton();
		
		IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();

		tbm.add(sort);
		tbm.add(filter);
//		tbm.add(advfilter);
		tbm.add(searchRecord);
		tbm.add(pageUp);
		tbm.add(pageDown);
		tbm.add(goToPage);
		tbm.add(exportRecords);
		
		// requires update because the toolbar control has been created by this point,
		// but the toolbar manager does not update it automatically once it has been created
		getViewSite().getActionBars().updateActionBars();
	}
	private Action createSearchButton() {
		String STR_FIND_RECORD =
			LogUIPlugin.getResourceBundle().getString("STR_FIND_RECORD");
		Action searchRecord = new Action(STR_FIND_RECORD) {
			public void run() {
				find();
			}
		};
		searchRecord.setText(STR_FIND_RECORD);
		LogUIPluginImages.setImageDescriptors(
			searchRecord,
			LogUIPluginImages.T_LCL,
			LogUIPluginImages.IMG_SEARCH_RECORD);
		searchRecord.setDescription(STR_FIND_RECORD);
		searchRecord.setToolTipText(STR_FIND_RECORD);
		return searchRecord;
	}

	private Action createPageUpButton() {
		String STR_PAGE_UP =
			LogUIPlugin.getResourceBundle().getString("STR_PAGE_UP");
		pageUp = new Action(STR_PAGE_UP) {
			public void run() {
				final IPage page = getCurrentPage();
				
				if (page != null && page instanceof LogPage) {

					org
					.eclipse
					.swt
					.custom
					.BusyIndicator
					.showWhile(null, new Runnable() {
						public void run() {

							LogContentProvider logContentProvider = (LogContentProvider)((LogPage) page).getView().getViewer().getContentProvider();
							pageUp.setEnabled(logContentProvider.pageUp());
							((LogPage) page).getView().update();
							
							setSelectionOnPageLoad(page,logContentProvider);
						}
					});
				}
			}
		};
		
		getViewSite().getActionBars().setGlobalActionHandler(ActionFactory.PREVIOUS.getId(),pageUp);
		
		pageUp.setText(STR_PAGE_UP);
		LogUIPluginImages.setImageDescriptors(
			pageUp,
			LogUIPluginImages.T_LCL,
			LogUIPluginImages.IMG_PAGE_UP);
		pageUp.setDescription(STR_PAGE_UP);
		pageUp.setToolTipText(STR_PAGE_UP);
		return pageUp;
	}
	
	private void setSelectionOnPageLoad(IPage page, LogContentProvider logContentProvider)
	{
		EObject mofObject = HyadesUtil.getObjectToView(HyadesUtil.getMofObject());							
		List elements = logContentProvider.getPageList(((LogPage) page).getView().getViewer().getInput());
		if(elements.size()>0 && ((LogPage) page).getView().getViewer().getSelection().isEmpty()){
			UIPlugin.getDefault().getSelectionModel(mofObject).add(elements.get(0));								
			((LogPage) page).getView().getViewer().setSelection(new StructuredSelection(elements.get(0)), true);
			notifySelectionChanged((EObject)elements.get(0));
		}
	}

	private Action createGoToPageButton() {
		String STR_GO_TO_PAGE =
			LogUIPlugin.getResourceString("STR_GO_TO_PAGE");
		goToPage = new Action(STR_GO_TO_PAGE) {
			public void run() {
				final IPage page = getCurrentPage();
				final LogContentProvider logContentProvider = (LogContentProvider)((LogPage) page).getView().getViewer().getContentProvider();
				final int totalPages = logContentProvider.getTotalPages();
				if(totalPages<2)
				{	
					goToPage.setEnabled(false);
					return;
				}
					
				String[] variables = new String[2];
				variables[0]="1";
				variables[1]=""+totalPages;
				String dialogTitle = LogUIPlugin.getResourceString("GO_TO_PAGE_DIALOG_TITLE");
				String dialogMessage = LogUIPlugin.getResourceString("GO_TO_PAGE_DIALOG_MESSAGE",variables);
				InputDialog inputDialog = new InputDialog(getSite().getShell(),dialogTitle,dialogMessage,null,new IInputValidator() {
					public String isValid(String newText) {
						if(newText==null || newText.length()==0)
							return "";
						try {
							int pageNr = Integer.parseInt(newText);
							if(pageNr<1 || pageNr>totalPages)
								return LogUIPlugin.getResourceString("GO_TO_PAGE_VNIR");
						} catch (Exception e) {
							return LogUIPlugin.getResourceString("GO_TO_PAGE_VNAN");
						}
						return null;
					}
				});
				inputDialog.open();
				if(inputDialog.getReturnCode() != InputDialog.OK)
					return;
				final int pageNr = Integer.parseInt(inputDialog.getValue());
				if (page != null && page instanceof LogPage) {

					org
					.eclipse
					.swt
					.custom
					.BusyIndicator
					.showWhile(null, new Runnable() {
						public void run() {
							
							boolean res = logContentProvider.goToPage(pageNr);
							if(pageNr==1)
								pageUp.setEnabled(false);
							goToPage.setEnabled(res);
							((LogPage) page).getView().update();
							
							setSelectionOnPageLoad(page,logContentProvider);							
						}
					});
				}
			}
		};
		
		goToPage.setText(STR_GO_TO_PAGE);
		getViewSite().getActionBars().setGlobalActionHandler(IWorkbenchActionConstants.M_NAVIGATE+"/goToPage",goToPage);
		
		LogUIPluginImages.setImageDescriptors(
			goToPage,
			LogUIPluginImages.T_LCL,
			LogUIPluginImages.IMG_GO_TO_PAGE);
		goToPage.setDescription(STR_GO_TO_PAGE);
		goToPage.setToolTipText(STR_GO_TO_PAGE);
		return goToPage;
	}
	
	private Action createPageDownButton() {
		String STR_PAGE_DOWN =
			LogUIPlugin.getResourceBundle().getString("STR_PAGE_DOWN");
		pageDown = new Action(STR_PAGE_DOWN) {
			public void run() {
				final IPage page = getCurrentPage();
				
				if (page != null && page instanceof LogPage) {

					org
					.eclipse
					.swt
					.custom
					.BusyIndicator
					.showWhile(null, new Runnable() {
						public void run() {
							
							LogContentProvider logContentProvider = (LogContentProvider)((LogPage) page).getView().getViewer().getContentProvider();
							pageDown.setEnabled(logContentProvider.pageDown());
							((LogPage) page).getView().update();
							
							setSelectionOnPageLoad(page,logContentProvider);
						}
					});
				}
			}
		};
		
		getViewSite().getActionBars().setGlobalActionHandler(ActionFactory.NEXT.getId(),pageDown);
		
		pageDown.setText(STR_PAGE_DOWN);
		LogUIPluginImages.setImageDescriptors(
			pageDown,
			LogUIPluginImages.T_LCL,
			LogUIPluginImages.IMG_PAGE_DOWN);
		pageDown.setDescription(STR_PAGE_DOWN);
		pageDown.setToolTipText(STR_PAGE_DOWN);
		return pageDown;
	}
	
	/**
	 * @see WorkbenchPart#setFocus 
	 * Creation date: (10/5/2001 11:42:53 AM)
	 */
	public void setFocus() {

		if (getCurrentPage() != null)
			getCurrentPage().setFocus();
	}

	public boolean isHandlingRefresh(){
		return handleRefreshEvent || filterRefreshEvent;
	}
	
	/**
	 * @return true if the object can be viewed using this type of view 
	 * 
	 */
	public boolean isValidObject(Object mofObject) {
		if (mofObject == null)
			return false;

		if (mofObject instanceof TRCAgent
			&& ((TRCAgent) mofObject).getType().equals(
				HyadesConstants.LOG_AGENT_TYPE)) {
			return true;
		}

		if (mofObject instanceof TRCProcessProxy) {
			return hasProcessLogAgents((TRCProcessProxy)mofObject);
		}
		
		if(mofObject instanceof TRCNode){
			return hasNodeLogAgents((TRCNode)mofObject);
		}
		
		if(mofObject instanceof TRCMonitor){
			return hasMonitorLogAgents((TRCMonitor)mofObject);
		}
		

		return true;
	}

	/**
	 * @return the object that should be viewed
	 * For example, if the process node is selected, the view should display
	 * the profiling data if trace collects profiling data
	 * Returns the agent if there is only one log agent
	 * 
	 */

	public EObject getObjectToView(EObject selObject) {

		if (selObject == null)
			return selObject;

		List list =null;
		
		if(selObject instanceof TRCMonitor){

            return selObject; //selection model must be at the monitor level
			//list = getLogObjectInMonitor((TRCMonitor)selObject);
			
		}else if(selObject instanceof TRCNode){
			
			return selObject;//selection model must be at the monitor level
			//list = getLogObjectInNode((TRCNode)selObject);
		
		}else if (selObject instanceof TRCProcessProxy) {

			list = getLogAgentInProcess((TRCProcessProxy)selObject); 
			
		}
		if(list!=null && list.size()==1){
			return (EObject)list.get(0);
		}
		return selObject;
	}

	private List getLogObjectInMonitor(EObject selObject){

		EList nodes = ((TRCMonitor)selObject).getNodes();
		TRCNode node = null;

		int size = nodes.size();
		int nrOfNodes = 0;
		List object_list = new ArrayList();
		List list;		

		for(int i=0; i < size && nrOfNodes < 2;i++){		
			node = (TRCNode)nodes.get(i);
			if(node==null){
				continue;
			}else{	
				
				list = getLogObjectInNode(node); 
				if(list.size()>=1){
					nrOfNodes++;					
					object_list.add(node);
				}
			}
			
		}
		
		if(object_list.size()==1){
			list = getLogObjectInNode((TRCNode)object_list.get(0));
			if(list.size()==1){
				object_list.clear();
				object_list.add(list.get(0));
			}
		}
		
		return object_list;
	}
	
	private List getLogObjectInNode(EObject selObject){

		EList processes = ((TRCNode)selObject).getProcessProxies();
		TRCProcessProxy process = null;
		int nrOfProcesses = 0;
		int size = processes.size();
		List list;
		List object_list = new ArrayList();

		for(int i=0; i < size && nrOfProcesses < 2;i++){		
			process = (TRCProcessProxy)processes.get(i);
			if(process!=null){
				list = getLogAgentInProcess(process);
				if(list.size()>=1){
					nrOfProcesses++;
					object_list.add(process);
				}
			}else{
				continue;								
			}
		}
		
		if(object_list.size()==1){
			list = getLogAgentInProcess((TRCProcessProxy)object_list.get(0));
			if(list.size()==1){
				object_list.clear();
				object_list.add(list.get(0));
			}
			
		}
		return object_list;
	}
	
	private List getLogAgentInProcess(EObject selObject){

		int nrOfAgents = 0;
		EList agents = ((TRCProcessProxy)selObject).getAgentProxies();
		int size = agents.size();
		TRCAgentProxy a = null;
		List list =  new ArrayList();
		
		for (int idx = 0; idx < size && nrOfAgents < 2; idx++) {
			a = (TRCAgentProxy) agents.get(idx);
			if (a == null || a.eIsProxy())
				continue;
			if (a.getType().equals(HyadesConstants.LOG_AGENT_TYPE)) {
				nrOfAgents++;
				list.add(a);
			}
		}
		
		return list;
	
	}
	
	
	private boolean hasMonitorLogAgents(TRCMonitor monitor){
		
		EList nodes = monitor.getNodes();
		TRCNode node = null;
		int size = nodes.size();
		boolean ret = false;		

		for(int i=0; i < size  && !ret;i++){		
			node = (TRCNode)nodes.get(i);
			if(node!=null){
				ret = hasNodeLogAgents(node);
			}
			
		}
		
		return ret;
		
	}
	
	private boolean hasNodeLogAgents(TRCNode node){

		EList processes = node.getProcessProxies();
		TRCProcessProxy process = null;
		boolean ret = false;
		
		int size = processes.size();

		for(int i=0; i < size && !ret;i++){
			process = (TRCProcessProxy)processes.get(i);
			if(process!=null){
				ret = hasProcessLogAgents(process);
			}
		}
		
		return ret;
		
	}
	
	private boolean hasProcessLogAgents(TRCProcessProxy process){
		
		EList agents = process.getAgentProxies();
		int size = agents.size();
		TRCAgentProxy agent = null;		
		
		for (int idx = 0; idx < size; idx++) {
			agent = (TRCAgentProxy) agents.get(idx);
			if (agent == null || agent.eIsProxy())
				continue;
			if (agent.getType().equals(HyadesConstants.LOG_AGENT_TYPE)) {
				return true;
			}
		}
		
		return false;
	}
	
	
	/**
	 * Initial view title 
	 */
	public String getViewTitle() {
		return _title;
	}

	public void setRecordSelection(final EObject record, EObject treeSel) {

		final LogViewerUI ui = logPage.getView();
		
		if (ui != null && ui.getViewer() != null) {
			//final LogContentProvider logContentProvider = (LogContentProvider)ui.getViewer().getContentProvider();
	   		BusyIndicator.showWhile(Display.getDefault(), new Runnable(){	
				public void run(){
					//if(logContentProvider.revealObject(record))
					//	ui.update();
					ui.revealObject(record);
				}			
			});	

			//ui.getViewer().setSelection(new StructuredSelection(record), true);
		}
	}

	protected void setViewTitle(Object selection) {
		if (selection != null) {
			/*if (selection instanceof EObject)
				selection = getObjectToView((EObject) selection);*/

			if (selection instanceof TRCAgentProxy) {
				TRCAgentProxy a = (TRCAgentProxy) selection;

				String status = HyadesUtil.getAgentLabel(a);

				if(a.getProcessProxy()!=null)
					setPartName
						((new StringBuffer())
							.append(_title)
							.append(" - ")
							.append(a.getName())
							//.append(" ")
							//.append(a.getProcessProxy().getName())
							.toString());
				return;

			}
		}
		setPartName(getViewTitle());
	}

	/**
	 * @return Returns the pageDown.
	 */
	public Action getPageDownAction() {
		return pageDown;
	}

	/**
	 * @return Returns the pageUp.
	 */
	public Action getPageUpAction() {
		return pageUp;
	}
	/**
	 * @return Returns the goToPage.
	 */
	public Action getGotPageAction() {
		return goToPage;
	}
	
	public void find(){
		searchDialog =	new SearchRecordDialog(
											getViewSite().getShell(),
											LogUIPlugin.getResourceString("STR_ST_FIND_SETTINGS"),
											null,
											this);
		
		initializeRecordSearchEngine();
		searchDialog.open();
	}
	
	public void filter(){
		FiltersDialog dialog = new FiltersDialog(
										getViewSite().getShell(),
										LogUIPlugin.getResourceString("STR_ST_FILTER_SETTINGS"),
										null,
										this,
										false);
		dialog.open();
		
	}
	
	public boolean findRecord(LogFindCriteria criteria){		
		
		Object sel = null;		
		LogRecordSearch searchUtil = getRecordSearchEngine();
		if(searchUtil.getType()==LogUIConstants.XMI){
			searchUtil.initStartNode(getStartNode(), LogUIConstants.XMI);
			sel = getRecordSearchEngine().search(criteria.getFilters(),criteria.isForwardDirection()?1:-1,lastMatch);
		}else if(searchUtil.getType()==LogUIConstants.SQL){
			Object inputObject = logPage.getView().getViewer().getInput();
			SimpleSearchQuery query = (SimpleSearchQuery)((LogContentProvider)logPage.getView().getViewer().getContentProvider()).getQuery();			
			if(query!=null){			
				sel = searchUtil.search(query, criteria);
			}else{
				ILogFilterCriteria filterCriteria = ((LogContentProvider)logPage.getView().getViewer().getContentProvider()).getFilterCriteria();
				sel = searchUtil.search((SimpleSearchQuery)LogQueryBuilder.createQuery((List)HyadesUtil.getLogAgents((EObject)inputObject), filterCriteria), criteria);
			}
		}

		setSelection(sel);
		return true;
	}

	public boolean filterRecord(ILogFilterCriteria criteria){
		LogUIPlugin.getDefault().fireRecordChangeNotification(new RecordChangeEvent(RecordChangeEvent.FILTER_ADV_CHANGE, this));
		return true;
	}
	

	public LogRecordSearch getRecordSearchEngine(){
		if(logRecordSearch==null){
			logRecordSearch = new LogRecordSearch();
			initializeRecordSearchEngine();
		}
		return logRecordSearch;	 
	}
	
	protected void initializeRecordSearchEngine(){
		
		ILogFilterCriteria filterCriteria = ((LogContentProvider)logPage.getView().getViewer().getContentProvider()).getFilterCriteria();
		
		Object inputObject = logPage.getView().getViewer().getInput();
		if(!(inputObject instanceof EObject)){
			return;
		}		
		
		LogViewerUI ui = logPage.getView();
		if (ui != null && ui.getViewer() != null) {
			LogContentProvider logContentProvider = (LogContentProvider)ui.getViewer().getContentProvider();
			cbeList = logContentProvider.getCachedElements();
		}
	
		Object start = getStartNode();
		
		if(logRecordSearch==null)
			logRecordSearch = new LogRecordSearch();
		
		if(isInputXMI((EObject)inputObject)){	
			logRecordSearch.initialize(cbeList, start, LogUIConstants.XMI);
		}
		else{
			logRecordSearch.initialize(cbeList, start, LogUIConstants.SQL) ;
		}
		
	}
	
	private void setSelection(Object sel){	
		if(sel==null){
			searchDialog.updateMessage(LogUIPlugin.getResourceString("STR_STRING_NOT_FOUND"));
			return;
		}
		
		//EObject mofObject = logPage.getTraceViewer().getObjectToView(HyadesUtil.getMofObject());
		if(sel!=null)
			setRecordSelection((EObject)sel, null);	            	

		lastMatch = sel;
		logPage.getView().getViewer().reveal(lastMatch);
		notifySelectionChanged((EObject)sel);
	}
	
	private void notifySelectionChanged(EObject sel) {
		LogUtil.notifyViewSelectionChanged(logPage.getView(),sel);
//		UIPlugin.getDefault().getSelectionModel(HyadesUtil.getMofObject()).add(sel);
//			
//		ViewSelectionChangedEvent event = UIPlugin.getDefault().getViewSelectionChangedEvent();
//		event.setSource(this);
//		//event.setSource(e.item.getData());
//		UIPlugin.getDefault().notifyViewSelectionChangedListener(event);
	}
	
	private Object getStartNode(){
			IStructuredSelection sel = (IStructuredSelection)((logPage).getView().getViewer().getSelection());
	
				Object object = sel.getFirstElement();;
				if (object instanceof CBECommonBaseEvent) {
					return object;
				}
				else if (object instanceof EObject) {
					return getCBENode(object);
				}
				else if (object instanceof FeatureNode) {
					return getCBENode(((FeatureNode)object).getElement());
				}else if(object instanceof TreeNode){
					return getCBENode(((TreeNode)object).getParent());
				}
				else if (object instanceof TerminalNode) {
					return getCBENode(((TerminalNode)object).getTreeParent());
				}

			return null;	
	}
	
	private Object getCBENode(Object object)
	{
		if (object instanceof CBECommonBaseEvent){
			return object;
		}
		else if (object instanceof EObject) {
			return getCBENode(((EObject)object).eContainer());
		}
		return null;
	}
	
	private boolean isInputXMI(EObject inputObject){

		List agentProxies = (List)HyadesUtil.getLogAgents((EObject)inputObject);
		TRCAgentProxy proxy = null;
		TRCAgent agent = null;
		int s = agentProxies.size();
		
		for(int i=0;i<s;i++){
			proxy = (TRCAgentProxy)agentProxies.get(i);
			agent = (TRCAgent)proxy.eGet(HierarchyPackage.eINSTANCE.getTRCAgentProxy_Agent(), false);
			if(agent==null){
				continue;				
			}
			if(EcoreUtil.getURI(agent).path().endsWith("xmi"))
			{				
				return true;				
			}			
		}
		return false;
		
	}
	/* (non-Javadoc)
	 * @see org.eclipse.hyades.log.ui.internal.actions.provider.ILogFindProvider#storeFindOptions(org.eclipse.hyades.log.ui.internal.actions.provider.LogFindCriteria)
	 */
	public boolean storeFindOptions(LogFindCriteria criteria) {
		FilterTableElement.saveFilters(criteria.getFilters(),LogUIConstants.PD_SEARCH_FILTER_OPTIONS,LogUIPlugin.getDefault().getPreferenceStore());
		LogUIPlugin.getDefault().getPreferenceStore().setValue(LogUIConstants.PD_SEARCH_DIRECTION_OPTIONS, criteria.isForwardDirection()?"1":"-1");
		return true;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.hyades.log.ui.internal.actions.provider.ILogFindProvider#loadFindPreferenceString()
	 */
	public String loadFindPreferenceString() {
		IPreferenceStore store = LogUIPlugin.getDefault().getPreferenceStore();
		return store.getString(LogUIConstants.PD_SEARCH_FILTER_OPTIONS);
	}
	
	public boolean loadFindDirection() {
		return LogUIPlugin.getDefault().getPreferenceStore().getString(LogUIConstants.PD_SEARCH_DIRECTION_OPTIONS).equals("1");
	}

	public LogPage getLogPage() {
		return logPage;
	}
	
	private void createAssociationsMap(Object resource) {
		if (resource instanceof CorrelationContainerProxy)
		{
			CorrelationEngine ce = ((CorrelationContainerProxy)resource).getCorrelationEngine();
			if (ce!=null)
			{
				AssociationEngine asoE = CbeEmfUtility.createAssociationEngine(ce);
				associationMap.put((CorrelationContainerProxy)resource,asoE);
				return;
			}				
		}
		if (resource instanceof TRCMonitor) {
			Object[] nodes = ((TRCMonitor) resource).getNodes().toArray();
			for (int i = 0; i < nodes.length; i++) {
				createAssociationsMap((TRCNode) nodes[i]);
			}
		}
		if (resource instanceof TRCNode) {
			Object[] processes = ((TRCNode) resource).getProcessProxies().toArray();
			for (int i = 0; i < processes.length; i++) {
				createAssociationsMap((TRCProcessProxy) processes[i]);
			}
		}
		if (resource instanceof TRCProcessProxy) {
			Object[] agents = ((TRCProcessProxy) resource).getAgentProxies().toArray();
			for (int i = 0; i < agents.length; i++) {
				createAssociationsMap((TRCAgentProxy) agents[i]);
			}
		}
		if (resource instanceof TRCAgentProxy) {
			Object[] correlationContainerProxys = ((TRCAgentProxy)resource).getInternalCorrelations().toArray();
			for(int i=0;i<correlationContainerProxys.length ;i++){
				createAssociationsMap((CorrelationContainerProxy)correlationContainerProxys[i]);
			}
		}
	}
		
	protected void exportRecords() {
		
		FileDialog dialog= new FileDialog(getCurrentPage().getControl().getShell(), SWT.SINGLE);
		dialog.setText(LogUIPlugin.getResourceString("STR_EXPORT_RECORD")); //$NON-NLS-1$
		dialog.setFilterExtensions(new String[] {"*.html", "*.htm"});
		String name = dialog.open();

		if (name != null) {
			exportToXML(name);
		}		
	}
		
	protected void exportToXML(String fileName) {
		List recordList = null;
		associationMap = new Hashtable();
		LogViewerUI ui = logPage.getView();
		if (ui != null && ui.getViewer() != null) {
			LogContentProvider logContentProvider = (LogContentProvider)ui.getViewer().getContentProvider();
			recordList = logContentProvider.getCachedElements();
		}
		
		Object selection = ui.getViewer().getInput();
			
		//creation association for any internal correlation for correlation container proxy
		if (selection instanceof CorrelationContainerProxy)
		{
			CorrelationContainerProxy proxy = (CorrelationContainerProxy)selection;
			Object[] agentProxys = proxy.getCorrelatedAgents().toArray();
			for (int i=1;i<agentProxys.length;i++) {
				createAssociationsMap((TRCAgentProxy)agentProxys[i]);
			}
		}
		
		//create any other correlation for resource
		createAssociationsMap(selection);
		
		saveToFile(fileName,recordList,null);
	}
	
	public void saveToFile(String filePath, List events, int index[]) {
	
		// Place file in user's directory, create required paths
		File path = new File(new File(filePath).getParent());
		path.mkdirs();
			
		// Open the file for writing
		PrintWriter file = null;
		try {
			file = new PrintWriter(new FileOutputStream(filePath));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		}
				
		if (file == null) {
			return;
		}
				
		file.println(LogUIConstants.CONST_CBES_HEADER);
				
		// Write out the Association Engines
		if (associationMap!=null && associationMap.size()>0) {
			for (Iterator i = associationMap.values().iterator(); i.hasNext();) {
				AssociationEngine asoE = (AssociationEngine)(i.next());
				String asoEString = EventFormatter.toCanonicalXMLString(asoE);
				file.println(asoEString);
			}
		}
				
		// Write out the CBEs
		if (events!=null) {
			
			for (int idx=0; idx<events.size(); idx++)
			{
				Object event = events.get(idx);
				if (event instanceof CBECommonBaseEvent) {
					CBECommonBaseEvent cbe = (CBECommonBaseEvent)event;
					CommonBaseEvent cbe2 = CbeEmfUtility.convertCbe(cbe,associationMap);
					String cbeString = EventFormatter.toCanonicalXMLString(cbe2);
					file.println(cbeString);
				}
			}
		}
				
		file.println(LogUIConstants.CONST_CBES_FOOTER);
		
		file.close();
	}

	/**
	 * Add a page to the workbook. If the mofObject is null, show default empty page
	 */
	public void addViewPage(EObject object) {
		this.prevEObject = object;
		
		boolean oldValue = handleRefreshEvent;

		//handleRefreshEvent = true;
		super.addViewPage(this.prevEObject);

		handleRefreshEvent = oldValue;
	}		

	/**
	 * Refresh the visible page when the view has become visible
	 *  
	 */
	protected void refreshVisiblePage() {
		addViewPage(this.prevEObject);
		EObject sel = getObjectToView(this.prevEObject);
		
		filterRefreshEvent = false;
				
		refreshPage(sel);

		selectionChanged(sel);
	}

	/*
	 * Selection has changed in the Monitors view (PDProjectExplorer)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		boolean openView = ((INavigator) part).isLinkingEnabled();
		Object obj = null;
		if (selection != null && !selection.isEmpty() && (selection instanceof IStructuredSelection)) {
			obj = ((IStructuredSelection) selection).getFirstElement();
		}

		if (openView && obj != null && (obj instanceof EObject)){
			this.prevEObject = (EObject) obj;
			fRefreshView = true;
			if (fPartVisible)
				refreshVisiblePage();
		}
		else
			super.selectionChanged(part, selection);
	}
	
	public String[] getFilterScope()
	{
		String[] scope = new String[1];
		scope[0] = HyadesConstants.LOGGING_AGENT_FILTER_SCOPE;
		return scope;
	}	
	
	public void filterUpdated()
	{
		if (fPartVisible)
			refresh();
		else
		{
			fForceRefresh = true;
			filterRefreshEvent = true;			
		}
	}
	
	private List getFilteredElements (SimpleSearchQuery filter, List source, EClass output) {
		if (source == null)
			return new ArrayList();
		
		if (filter == null)
			return new ArrayList();
		
		filter.getSources().clear();
		
		Resource resource = null;
		
		for (int i = 0; i < source.size(); i++)
		{
			if (source.get(i) instanceof EObject)
			{
				filter.getSources().add(EcoreUtil.getURI((EObject)source.get(i)).toString());
				
				if (resource == null)
					resource = ((EObject)source.get(i)).eResource(); 
			}
		}
		
		if (resource == null)
			return new ArrayList();
			
		filter.getOutputElements().clear();
		SimpleOperand outputElement = ExtensionsFactory.eINSTANCE.createSimpleOperand();
		outputElement.setType(output);
		filter.getOutputElements().add(outputElement);
		
		HierarchyXMIQueryEngine engine = new HierarchyXMIQueryEngine();
		QueryResult result = engine.executeQuery(filter, resource.getResourceSet(),Collections.EMPTY_LIST);
		if(result==null)
			return new ArrayList();
		return (List)((ResultEntry)result.getResultEntries().get(0)).getValue();
	}	
	
	
	protected List getFilteredLogRecords (List source) {
		SimpleSearchQuery filter = getCurrentFilter();	
		if (filter == null)
		{
			filter = QueryUtils.getEmptyQuery();
		}
		
		return getFilteredElements(filter, source, CBEPackageImpl.init().getCBECommonBaseEvent());  
	}
	
	
}
