package org.eclipse.hyades.logging.adapter.internal.util;


import org.eclipse.hyades.logging.adapter.AdapterException;
import org.eclipse.hyades.logging.adapter.AdapterPlugin;
import org.eclipse.hyades.logging.adapter.IComponent;
import org.eclipse.hyades.logging.adapter.parsers.PreparationException;
import org.eclipse.hyades.logging.adapter.util.Messages;

/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ComponentLoaderUtil.java,v 1.10 2005/03/23 08:09:06 dnsmith Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
public class ComponentLoaderUtil
{
	/**
	 * Instatiate an object and guarantee its class.
	 * @param className The name of the class to instantiate.
	 * @param impls The name of the class it subclases.
	 **/
	public static Object instantiate(String className, String impls) throws PreparationException  {
	    try	{
	        /* First we will assume we are running inside of Eclipse. Look at our extensions to see 
			 * if anyone has provided this class in another plugin.
			 */
	        IComponent component = AdapterPlugin.getComponent(className);
			
			if(component != null){
			    return component;
			}
		}
	    catch(AdapterException e) {
	    	throw new PreparationException(Messages.getString("HyadesGAInstantiation_ERROR_", className),e);
	    }
		catch(Throwable  e) {
			// We must not be running in Eclipse environment so ignore this exception and continue
		}
		
		// Try to load the class from the classpath
		
		Class c;
		try {
		    c = Class.forName(impls);
		}
		catch(ClassNotFoundException eprime) {
			throw new PreparationException(Messages.getString("HyadesGAClassLoad_ERROR_", impls));
		}
		
		try {
				
			Class o = Class.forName(className);
			Object res = null;
			
			try {
			    
			    //Attempt to create an instance of the class using the no-argument constructor:
			    res = o.newInstance();
            } 
			catch (Throwable t) {
			    
			    //If the no-argument constructor of the class is privatized, attempt 
			    //to retrieve the reference to the singleton instance of the class:
			    res = o.getMethod("getInstance", null).invoke(null, null);
            }
						
			if (c.isInstance(res)) {
				return res;
			}
			else {
				throw new PreparationException(Messages.getString("HyadesGAInitialization_Invalid_Class_Failure_ERROR_", className, impls));
			}
		}
		catch(ClassNotFoundException eprime) {
			throw new PreparationException(Messages.getString("HyadesGAClassLoad_ERROR_", className));
		}
		catch (IllegalAccessException eprime) {
			throw new PreparationException(Messages.getString("HyadesGAAccess_ERROR_", className));
		}
		catch (Throwable eprime) {
			throw new PreparationException(Messages.getString("HyadesGAInstantiation_ERROR_", className),eprime);
		}
	}
}