/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: CBELogOutputter.java,v 1.17 2005/03/23 08:09:06 dnsmith Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.logging.adapter.outputters;

import java.util.Hashtable;

import org.eclipse.hyades.logging.adapter.AdapterInvalidConfig;
import org.eclipse.hyades.logging.adapter.IOutputter;
import org.eclipse.hyades.logging.adapter.impl.Outputter;
import org.eclipse.hyades.logging.adapter.util.Messages;
import org.eclipse.hyades.logging.core.LoggingAgent;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.util.EventFormatter;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author smith
 *
 * This outputter writes CBE's to a logging agent
 * 
 */
public class CBELogOutputter extends Outputter implements IOutputter {
	private LoggingAgent cbelogger = null;
	
	private static String defaultAgentName = "GenericAdapterCBELogger";
	
	private String agentName = null;
	
	private int bufferSize = 100; 
	
	private long waitTime = 0;
	
	/* (non-Javadoc)
	 * @see org.eclipse.hyades.logging.adapter.IProcessUnit#processEventItems(java.lang.Object[])
	 */
	public Object[] processEventItems(Object[] msgs) {
		if (msgs == null)
			return null;
		return processCBEs((CommonBaseEvent[]) msgs);
	}

	/**
	 * Process the CBE events passed in.
	 * @param events CBE events to process
	 * @return CommonBaseEvent[]
	 */
	private CommonBaseEvent[] processCBEs(CommonBaseEvent[] events)
	{
		int cbesReturned = 0;
		
		// If the agent is not being monitored then wait for it to be so
		
		if (events.length > 0 && !cbelogger.isLogging() && getWaitTime() > 0) {

		    CommonBaseEvent event = getEventFactory().createCommonBaseEvent();

		    event.getMsgDataElement().setMsgCatalogId("HyadesGA_CBE_Logging_Agent_Waiting_INFO_");
		    event.getMsgDataElement().setMsgCatalogTokensAsStrings(new String[]{getUniqueID(),getAgentName()});							    

		    event.setSeverity(CommonBaseEvent.SEVERITY_INFORMATION);
			
		    log(event);
			
		    while ( !cbelogger.waitUntilLogging(getWaitTime())) {

			    event = getEventFactory().createCommonBaseEvent();
			   
			    event.getMsgDataElement().setMsgCatalogId("HyadesGA_CBE_Logging_Agent_Waiting_INFO_");
			    event.getMsgDataElement().setMsgCatalogTokensAsStrings(new String[]{getUniqueID(),getAgentName()});							    

			    event.setSeverity(CommonBaseEvent.SEVERITY_INFORMATION);

			    log(event);
			} 
			
/*          This does not work - waitUntilLogging is not returning true after user attaches to 
			agent but it should. 
 
			if (!cbelogger.waitUntilLogging(60000)) {
				if (!cbelogger.isLogging()) {
				return events;
				}
			}
*/			
		}
		
		for (int i = 0; i < events.length; i++)
		{
			if (events[i] != null) {
								
				try {
					// Write CBE to logging agent
					cbelogger.write(EventFormatter.toCanonicalXMLString(events[i],false));
					incrementItemsProcessedCount();
					cbesReturned++;
				} catch (Exception ioe) {					

				    CommonBaseEvent event = getEventFactory().createCommonBaseEvent();

				    event.getMsgDataElement().setMsgCatalogId("HyadesGA_CBE_Logging_Agent_Write_ERROR_");
				    event.getMsgDataElement().setMsgCatalogTokensAsStrings(new String[]{getUniqueID(),getAgentName()});							    

				    event.setSeverity(CommonBaseEvent.SEVERITY_FATAL);
					
				    log(event);
				}
			}
		}
		trace("HyadesGATrace_Component_Processing_End_INFO_", getName(), Integer.toString(cbesReturned));
		return events;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.hyades.logging.adapter.IProcessUnit#testProcessEventItems(java.lang.Object[])
	 */
	public Object[] testProcessEventItems(Object[] msgs)
		throws AdapterInvalidConfig {
			if(!(msgs instanceof CommonBaseEvent[])) {
				throw new AdapterInvalidConfig("This outputter will only accept arrays of CommonBaseEvent");
			}
			return testProcessCBEs((CommonBaseEvent[]) msgs);
		}
		
	private CommonBaseEvent[] testProcessCBEs(CommonBaseEvent[] events)
		{
			return events;
		}


	/**
	 * update the configuration based on the configuration Element
	 */
	public void update() throws AdapterInvalidConfig {
		
		// first get the basic configuration set
		super.update();
		
		// Get configuration
		
		Element element = getConfiguration();
		
		Element outputterTypeInstance = null;
		Element outputterNode;
		
		String waittm = null;
		String agent = null;

		// Get the outputter parameters from the outputter properties
		Hashtable outputterProperties = getProperties();		
		if (outputterProperties != null && !outputterProperties.isEmpty()) {
			waittm = (String)outputterProperties.get(Messages.getString("HyadesGAwaitTimeAttributeName"));
			if (waittm != null && waittm.length() > 0) {
				setWaitTime(Long.valueOf(waittm).longValue());
			}

			setAgentName((String)outputterProperties.get(Messages.getString("HyadesGAagentNameAttributeName")));
			
			/* Retrieve our buffer size from the property list.  */
			String bufSizeStr = (String)outputterProperties.get(Messages.getString("HyadesGAbufferSizeAttributeName"));
			if (bufSizeStr != null) {
				try {
					int bufSize=Integer.valueOf(bufSizeStr).intValue();
					setBufferSize(bufSize);
				}
				catch(NumberFormatException e) {
					/* We have a buffer size which is not an integer.  Log the fact we are using the default size */
					CommonBaseEvent event = getEventFactory().createCommonBaseEvent();

				    event.getMsgDataElement().setMsgCatalogId("HyadesGA_CBE_Logging_Agent_Write_ERROR_");
				    event.getMsgDataElement().setMsgCatalogTokensAsStrings(new String[]{getUniqueID(),getAgentName(), "100"});							    

				    event.setSeverity(CommonBaseEvent.SEVERITY_WARNING);
					
				    log(event);
				}
				
			}
		}
		else {		
			/**
			 * If there are no properties then get the outputter parameters from the old LogOutputterType instance
			 */
			NodeList outputterNodes = element.getChildNodes();
			for (int i = 0; i < outputterNodes.getLength(); i++) {
				if (outputterNodes.item(i).getNodeType() == Node.ELEMENT_NODE) {
					outputterNode = (Element) outputterNodes.item(i);
	
					if (outputterNode.getTagName().equals(Messages.getString("HyadesGALogOutputterTypeTagName"))) {
						// Get the outputter parameters from the outputter type instance attributes
						outputterTypeInstance = outputterNode;
						// Get attributes		
						if (outputterTypeInstance.hasAttribute(Messages.getString("HyadesGAwaitTimeAttributeName"))) {
							String wtime = outputterTypeInstance.getAttribute(Messages.getString("HyadesGAwaitTimeAttributeName"));
							if (wtime != null && !wtime.equals("")) {
								setWaitTime(Long.valueOf(wtime).longValue());
							}
						}									
						if (outputterTypeInstance.hasAttribute(Messages.getString("HyadesGAagentNameAttributeName")))
							setAgentName(outputterTypeInstance.getAttribute(Messages.getString("HyadesGAagentNameAttributeName")));
						if (outputterTypeInstance.hasAttribute(Messages.getString("HyadesGAbufferSizeAttributeName"))) {
							/* Retrieve our buffer size from the property list.  */
							String bufSizeStr = outputterTypeInstance.getAttribute(Messages.getString("HyadesGAbufferSizeAttributeName"));
							if (bufSizeStr != null) {
								try {
									int bufSize=Integer.valueOf(bufSizeStr).intValue();
									setBufferSize(bufSize);
								}
								catch(NumberFormatException e) {
									/* We have a buffer size which is not an integer.  Log the fact we are using the default size */
									CommonBaseEvent event = getEventFactory().createCommonBaseEvent();

								    event.getMsgDataElement().setMsgCatalogId("HyadesGA_CBE_Logging_Agent_Write_ERROR_");
								    event.getMsgDataElement().setMsgCatalogTokensAsStrings(new String[]{getUniqueID(),getAgentName(), "100"});							    

								    event.setSeverity(CommonBaseEvent.SEVERITY_WARNING);
									
								    log(event);
								}
								
							}
						}
					}
				}
			}		
		}
		
		// Create the logging agent
		if (cbelogger == null) {
			String logAgentName;
			if (getAgentName() == null) {
				logAgentName = defaultAgentName;
			}
			else {
				logAgentName = getAgentName();
			}
			cbelogger = new LoggingAgent(logAgentName,bufferSize);
		}

	}

	/**
	 * Sets the agent name
	 * @param aName The agent name to set
	 */

	public void setAgentName(String aName) {
		agentName = aName;
	}

	/**
	 * Gets the agent name
	 * @return String
	 */

	public String getAgentName() {
		return agentName;
	}
	
	/**
	 * Sets the buffer size of the logging agent (i.e., the number of records buffered
	 * while the logging agent is not connected to the RAC). 
	 * @param bufSize
	 */
	public void setBufferSize(int bufSize) {
		bufferSize = bufSize; 
	}
	
	/** 
	 * Gets the buffer size (i.e., the number of records buffered while the logging agent
	 * is not connected to the RAC). 
	 * @return the buffer size (as an int)
	 */
	public int getBufferSize() {
		return bufferSize; 
	}
	

	/**
	 * Sets the wait time.
	 * @param wTime The wait time to set
	 */

	public void setWaitTime(long wTime) {
		waitTime = wTime;
	}

	/**
	 * @return
	 */
	/**
	 * Gets the wait time.
	 * @return long
	 */

	public long getWaitTime() {
		return waitTime;
	}

	public void stop() {
		cbelogger.deregister();
	}
}
