/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: SDbPlugin.java,v 1.13 2005/05/06 04:05:42 popescu Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.sdb.internal;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.hyades.models.internal.sdb.impl.SDBFactoryImpl;
import org.eclipse.hyades.models.internal.sdb.impl.SDBPackageImpl;
import org.eclipse.hyades.models.internal.sdb.impl.SDBResourceSetImpl;
import org.eclipse.hyades.sdb.internal.util.SdUIConstants;
import org.eclipse.hyades.ui.internal.util.StringUtil;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

public class SDbPlugin extends AbstractUIPlugin
{
	private static SDbPlugin inst;
	private static ResourceBundle fResourceBundle;
	private SDBFactoryImpl factory;
	private ResourceSet resourceSet;
	public static final String PLUGIN_ID = "org.eclipse.hyades.sdb"; //$NON-NLS-1$

	public SDbPlugin() {

		if (inst == null)
			inst = this;
		
	}
	public static SDbPlugin getDefault() {
		return inst;
	}

	public static String getPluginId() {
		return PLUGIN_ID;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/16/2001 4:57:29 PM)
	 * @return com.ibm.jface.resource.ImageDescriptor
	 * @param name java.lang.String
	 */
	public static ImageDescriptor getImageDescriptor(String name) {
		return SDbPluginImages.getImageDescriptor(name);
	}
	public static ResourceBundle getResourceBundle() {
	
		if(fResourceBundle == null)
		{
			try {
				fResourceBundle = Platform.getResourceBundle(Platform.getBundle(PLUGIN_ID));
			} catch (MissingResourceException e) {
				e.printStackTrace();
				fResourceBundle = null;
			}
		}
				
		return fResourceBundle;
	}
	
	public static String getResourceString(String key) {
		ResourceBundle bundle = SDbPlugin.getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}
	
	public static String getResourceString(String key, String[] variables) {
		String value = getResourceString(key);
		for(int i=variables.length; i>0; i--)
			value = StringUtil.replace(value, "%"+i, variables[i-1]);
		return value;
	}
	
	/* (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		
		super.stop(context);
		
		savePluginPreferences();
		SDbPluginImages.shutdown();
		
	}
	
	/* (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		
		SDBPackageImpl.init();
		factory = new SDBFactoryImpl();

		// init resourceSet
		resourceSet = SDBResourceSetImpl.getInstance();

		super.start(context);
	}

	public SDBFactoryImpl getSDBFactory() {
		if (factory == null)
			factory = new SDBFactoryImpl();
		return factory;
	}

	/*
	 * 
	 */
	public ResourceSet getResourceSet() {
		return resourceSet;
	}

	/**
	 * Sets default preference values. These values will be used
	 * until some preferences are actually set using Preference dialog.
	 */
	protected void initializeDefaultPreferences(IPreferenceStore store) {

		String url =
			getResourceString("STR_StdURL_NAME")
				+ "\t"
				+ SdUIConstants.StdURL
				+ "\n"
				+ getResourceString("STR_AdvURL_NAME")
				+ "\t"
				+ SdUIConstants.AdvURL
				+ "\n"
				+ getResourceString("STR_StdURL_NAME_V5")
				+ "\t"
				+ SdUIConstants.StdURL
				+ "\n"
				+ getResourceString("STR_StdURL_NAME_V6")
				+ "\t"
				+ SdUIConstants.StdURL
				+ "\n"
				+ getResourceString("STR_AdvURL_NAME_V5")
				+ "\t"
				+ SdUIConstants.AdvURL;

		store.setDefault(SdUIConstants.SYMPTOM_DB_URL, url);
		store.setDefault(SdUIConstants.SYMPTOM_DB_URL_32, url);
		
	}

	public void log(IStatus status) {
		getLog().log(status);
	}
	
	public void log(Throwable e) {
		log(new Status(IStatus.ERROR, PLUGIN_ID, IStatus.ERROR, "Error", e)); //$NON-NLS-1$
	}
	
}
