/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TestUIPlugin.java,v 1.18 2005/04/22 14:51:01 bjiang Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui;

import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.hyades.models.common.common.CMNNamedElement;
import org.eclipse.hyades.models.common.facades.behavioral.impl.DatapoolFacadeResourceFactoryImpl;
import org.eclipse.hyades.models.common.facades.behavioral.impl.FacadeResourceFactoryImpl;
import org.eclipse.hyades.models.common.testprofile.TPFExecutionEvent;
import org.eclipse.hyades.models.common.util.ICommonConstants;
import org.eclipse.hyades.test.ui.adapter.IResourceSetFileAdaptable;
import org.eclipse.hyades.test.ui.adapter.TestAdapterFactory;
import org.eclipse.hyades.test.ui.navigator.ITypedElementProxyNode;
import org.eclipse.hyades.ui.HyadesUIPlugin;
import org.eclipse.hyades.ui.internal.logicalfolder.LogicalFolder;
import org.eclipse.hyades.ui.internal.util.JavaUtil;
import org.eclipse.hyades.ui.internal.util.ResourceBundleManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * Hyades Test UI Plugin class.  This class should not be used outside the context of
 * Eclipse.
 * 
 * @author marcelop
 * @since 0.0.1
 */
public class TestUIPlugin extends AbstractUIPlugin
{
	public static final String PLUGIN_ID = "org.eclipse.hyades.test.ui"; //$NON-NLS-1$
	private static TestUIPlugin instance;
	private ResourceBundleManager resourceBundleManager;
	private static final String BUNDLE_NAME = "org.eclipse.hyades.test.ui.messages";//$NON-NLS-1$
	private ResourceBundle resourceBundle;
	
	/**
	 * Extension point id used to register the test case creation wizards.
	 */
	public static final String EP_TEST_CASE_WIZARD = "newTestCaseWizards"; //$NON-NLS-1$
	
	/**
	 * Extension point id used to register the generate wizards.
	 */
	public static final String EP_GENERATE_WIZARDS = "generateWizards"; //$NON-NLS-1$

	/**
	 * Extension point id used to register the run wizards.
	 */
	//public static final String EP_RUN_WIZARDS = "runWizards"; //$NON-NLS-1$

	/**
	 * Constructor for HyadesUIPlugin
	 * @param descriptor
	 */
	public TestUIPlugin()
	{
		super();		
		instance = this;
	}
	
	/**
	 * Returns the instance of this class created by the eclipse framework.
	 * @return HyadesUIPlugin
	 */
	public static TestUIPlugin getInstance()
	{
		return instance;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
	
		resourceBundleManager = new ResourceBundleManager(); 
		resourceBundleManager.add(getResourceBundle());
		resourceBundleManager.add(HyadesUIPlugin.getInstance().getResourceBundle());
		resourceBundleManager.add(ResourceBundle.getBundle(BUNDLE_NAME));
		
		Platform.getAdapterManager().registerAdapters(TestAdapterFactory.INSTANCE, IResourceSetFileAdaptable.class);
		Platform.getAdapterManager().registerAdapters(TestAdapterFactory.INSTANCE, IResource.class);
		Platform.getAdapterManager().registerAdapters(TestAdapterFactory.INSTANCE, LogicalFolder.class);
		Platform.getAdapterManager().registerAdapters(TestAdapterFactory.INSTANCE, CMNNamedElement.class);
		Platform.getAdapterManager().registerAdapters(TestAdapterFactory.INSTANCE, TPFExecutionEvent.class);
		Platform.getAdapterManager().registerAdapters(TestAdapterFactory.INSTANCE, ITypedElementProxyNode.class);
		Platform.getAdapterManager().registerAdapters(TestAdapterFactory.INSTANCE, Resource.class);

		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(ICommonConstants.TEST_SUITE_FILE_EXTENSION, new FacadeResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(ICommonConstants.DEPLOYMENT_FILE_EXTENSION, new FacadeResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(ICommonConstants.EXECUTION_FILE_EXTENSION, new FacadeResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(ICommonConstants.LOCATION_FILE_EXTENSION, new FacadeResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(ICommonConstants.DATAPOOL_FILE_EXTENSION, new DatapoolFacadeResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(ICommonConstants.ARTIFACT_FILE_EXTENSION, new FacadeResourceFactoryImpl());


		JavaUtil.addResourcePatternToNotCopyList("*." + ICommonConstants.TEST_SUITE_FILE_EXTENSION); //$NON-NLS-1$
		JavaUtil.addResourcePatternToNotCopyList("*." + ICommonConstants.DEPLOYMENT_FILE_EXTENSION); //$NON-NLS-1$
		JavaUtil.addResourcePatternToNotCopyList("*." + ICommonConstants.LOCATION_FILE_EXTENSION); //$NON-NLS-1$
		JavaUtil.addResourcePatternToNotCopyList("*." + ICommonConstants.EXECUTION_FILE_EXTENSION); //$NON-NLS-1$
		JavaUtil.addResourcePatternToNotCopyList("*." + ICommonConstants.DATAPOOL_FILE_EXTENSION); //$NON-NLS-1$
		JavaUtil.addResourcePatternToNotCopyList("*." + ICommonConstants.ARTIFACT_FILE_EXTENSION); //$NON-NLS-1$

		super.start(context);

		Runnable operation = new Runnable() {
			public void run() {
				try	{
					TestUIImages.INSTANCE.initialize(new URL(getBundle().getEntry("/"), "icons/full/"), getImageRegistry()); //$NON-NLS-1$
				}
				catch(Exception e) {
					logError(e);
				}		
			}
		};
		
		/*
		 * #63850: Platform discourages use of thread syncronization in
		 * start(). Using asyncExec() in the case where this is not
		 * already in the UI thread (not the common case).
		 */
		if (Display.getCurrent() == Display.getDefault()) {
			operation.run();
		}
		else {
			Display.getDefault().asyncExec(operation);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		resourceBundleManager.dispose();
		TestUIExtension.dispose();
		TestUI.dispose();

		savePluginPreferences();
		Platform.getAdapterManager().unregisterAdapters(TestAdapterFactory.INSTANCE);
		TestAdapterFactory.INSTANCE.dispose();
		
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().remove(ICommonConstants.TEST_SUITE_FILE_EXTENSION);
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().remove(ICommonConstants.DEPLOYMENT_FILE_EXTENSION);
		/**
		 * @author psun
		 */
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().remove(ICommonConstants.DATAPOOL_FILE_EXTENSION);
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().remove(ICommonConstants.LOCATION_FILE_EXTENSION);
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().remove(ICommonConstants.ARTIFACT_FILE_EXTENSION);
		
		instance = null;	
		super.stop(context);
	}
	
	/**
	 * Returns the resource bundle used by this plugin.
	 * 
	 * <p>IMPORTANT: Don't use this method to retrieve values from the
	 * resource bundle.  For this purpose use the static <code>getString()</code> 
	 * defined in this class.
	 * 
	 * <p>This method is provided so this resource bundle can
	 * be used as the parent of another resource bundle.
	 * 
	 * @return ResourceBundle
	 */
	public ResourceBundle getResourceBundle()
	{
		if(resourceBundle == null)
		{
		
			try
			{
				resourceBundle = Platform.getResourceBundle(Platform.getBundle(PLUGIN_ID));
			}
			catch (Exception e) {
				resourceBundle = null;
			}
		}
		
		return resourceBundle;
	}

	/**
	 * Returns this plugin's id.
	 * @return String
	 */
	public static String getID()
	{
		return PLUGIN_ID;
	}
	
	/**
	 * Logs an error described by a throwable.
	 * 
	 * <p>This method should be used whenever a class in this plugin
	 * has to log an error since it adheres to the global logging
	 * strategy.
	 * 
	 * @param throwable
	 */
	public static void logError(Throwable throwable)
	{
		Status status = new Status(IStatus.ERROR, getID(), 0, throwable.toString(), throwable);
		getInstance().getLog().log(status);
	}
	
	/**
	 * Logs an error described by a text.
	 * 
	 * <p>This method should be whenever a class in this plugin
	 * has to log an error since it adheres to the global logging
	 * strategy.
	 * 
	 * @param text
	 */
	public static void logError(String text)
	{
		logError(new Throwable(text));
	}
	
	/**
	 * Logs an information described by a text.
	 * 
	 * <p>This method should be whenever a class in this plugin
	 * has to log an information since it adheres to the global logging
	 * strategy.
	 * 
	 * @param text
	 */
	public static void logInfo(String text)
	{
		Status status = new Status(IStatus.INFO, getID(), 0, text, new Throwable(text));
		getInstance().getLog().log(status);
	}
	
	/**
	 * Returns the "plugin.properties" file's value associate to a given key.
	 * @param key
	 * @return String
	 * @throws java.util.MissingResourceException if the key is not in the file
	 * @throws NullPointerException if key is null
	 */
	public static String getString(String key)
	throws NullPointerException, MissingResourceException
	{
		return instance.resourceBundleManager.getString(key);
	}
	
	/**
	 * Returns the string value associate to a given key.  The key is passed to
	 * each resource bundle in the order they are retrieve by the 
	 * {@link #iterator()} method.
	 * 
	 * <p>The <code>arg</code> string defined replaces the %1 
	 * variable defined in the file's values.
	 * 
	 * <p>Example: If the value associated to the key <code>"a"</code> is 
	 * <code>"%0 %1 %2 %3 %4"</code> and arg is <code>"v1"</code>,
	 * the return of this method is <code>"%0 v1  %2 %3 %4"</code>.
	 * 
	 * @param key
	 * @param arg
	 * @return String
	 * @throws java.util.MissingResourceException if the key is not in the file
	 * @throws NullPointerException if key is null
	 */
	public static String getString(String key, String arg)
	throws NullPointerException, MissingResourceException
	{
		return instance.resourceBundleManager.getString(key, arg);
	}	

	/**
	 * Returns the "plugin.properties" file's value associate to a given key.
	 * 
	 * <p>The strings defined in <code>args</code> replaces the %n (where n>=1) 
	 * variables defined in the file's values.
	 * 
	 * <p>Example: If the value associated to the key <code>"a"</code> is 
	 * <code>"%0 %1 %2 %3 %4"</code> and args is <code>{"v1", null, "v3"}</code>,
	 * the return of this method is <code>"%0 v1  v3 %4"</code>.
	 * 
	 * @param key
	 * @param args
	 * @return String
	 * @throws java.util.MissingResourceException if the key is not in the file
	 * @throws NullPointerException if key is null
	 */
	public static String getString(String key, String[]args)
	throws NullPointerException, MissingResourceException
	{
		return instance.resourceBundleManager.getString(key, args);
	}
	
	/**
	 * Insert the method's description here.
	 * Creation date: (8/8/2001 5:11:57 PM)
	 * @param store org.eclipse.jface.preference.IPreferenceStore
	 */
	protected void initializeDefaultPreferences(IPreferenceStore store) {
		store.setDefault(TestUI.REPORT_OPEN_EDITOR, false);
		store.setDefault(TestUI.OPEN_EDITOR, true);
		store.setDefault(TestUI.SAVE_INTERVAL, 5000);			
		store.setDefault(TestUI.EXECUTION_FILTER, 3);	
		store.setDefault(TestUI.LOCALHOST_PORT, "10002");
		store.setDefault(TestUI.DEFAULT_HOST_NAME, "localhost");
	}
	
}
