/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: MoveFeatureChildrenUpAction.java,v 1.3 2005/02/16 22:22:04 qiyanli Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.action.model;

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.hyades.test.ui.TestUIImages;
import org.eclipse.hyades.test.ui.TestUIPlugin;
import org.eclipse.jface.viewers.IStructuredSelection;
;

/**
 * This class handles the move up action of elements that are {@link org.eclipse.emf.ecore.EStructuralFeature}
 * of an {@link org.eclipse.emf.ecore.EObject}.
 * 
 * @author bjiang
 * @since  3.0
 */
public class MoveFeatureChildrenUpAction extends FeatureChildrenSelectionAction
{
	/**
	 * 
	 */
	public MoveFeatureChildrenUpAction(EObject parent, EStructuralFeature feature)
	{
		super(TestUIPlugin.getString("LBL_UP"), parent, feature); //$NON-NLS-1$

		setToolTipText(TestUIPlugin.getString("ACT_UP_DSC")); //$NON-NLS-1$
		TestUIImages.INSTANCE.setImageDescriptors(this, TestUIImages.ACT_UP);
		
		if(!feature.isMany())
		    this.setEnabled(false);
	}
	
	/**
	 * All the children must belong to the same parent.
	 * @see org.eclipse.ui.actions.SelectionListenerAction#updateSelection(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	protected boolean updateSelection(IStructuredSelection selection)
	{
		if(!super.updateSelection(selection))
			return false;
		
		EList allChildren = (EList)getEStructuralFeatureValue();
		if(allChildren == null)
		    return false;
		if(allChildren.size() <= 1)
		{
			getChildren().clear();
			return false;
		}
		
		if((getChildren().size() == 1) && (allChildren.get(0) == getChildren().get(0)))
		{
			getChildren().clear();
			return false;			
		}
		
		return true;
	}

	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run()
	{
		setActionPerformed(false);

		EList allChildren = (EList)getEStructuralFeatureValue();
		if(allChildren == null || allChildren.size() <= 1)
			return;
		
		int top = 0;
		for(Iterator i=getChildren().iterator(); i.hasNext(); )
		{
			Object element = i.next();
			int index = allChildren.indexOf(element);
			if(index < 0)
				continue;
				
			if(index > top)
				allChildren.move(index, index-1);
			else
				top++;
		}		

		setActionPerformed(true);
	}
}
