/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ExecutionHTNProvider.java,v 1.6 2005/03/22 17:20:09 dguilbaud Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.internal.navigator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.hyades.models.common.testprofile.TPFExecutionEvent;
import org.eclipse.hyades.models.common.testprofile.TPFExecutionResult;
import org.eclipse.hyades.models.common.testprofile.TPFTestSuite;
import org.eclipse.hyades.models.common.util.ICommonConstants;
import org.eclipse.hyades.test.ui.IHyadesTestNavigatorProvider;
import org.eclipse.hyades.test.ui.TestUIImages;
import org.eclipse.hyades.test.ui.internal.model.EMFUtil;
import org.eclipse.hyades.test.ui.internal.model.EventUtil;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.emf.ecore.EObject;

/**
 * @author kdarcour
 */
public class ExecutionHTNProvider implements IHyadesTestNavigatorProvider, IExecutableExtension {

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement) 
	{
		if (parentElement instanceof IFile) 
		{
			IFile executionfile = (IFile) parentElement;
			String extension = executionfile.getFileExtension();
			if (extension.equals(ICommonConstants.EXECUTION_FILE_EXTENSION) )
			{
				EObject[] eobjects = EMFUtil.load(null, executionfile);
				// construction of the array
				int j = 0;
				for ( int i = 0; i < eobjects.length; i++ )
				{
					if ( eobjects[i] instanceof TPFExecutionResult )
					   j++;
				}
				Object[] root = new Object[j];
				if ( j != 0 )
				{
					j = 0;
					for ( int i = 0; i < eobjects.length; i++ )
					{
						if ( eobjects[i] instanceof TPFExecutionResult )
						{ 	
					   		root[j] = ((TPFExecutionResult) eobjects[i]);
					   		j++;
						}  
					}
				}
				return root;
			}
		}
		return new Object[0];
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) 
	{
		return getChildren(inputElement);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
	public Image getImage(Object element) 
	{
		if (element instanceof IFile) 
		{
			IFile file = (IFile) element;
			String ext = file.getFileExtension();
			if (ext.equals(ICommonConstants.EXECUTION_FILE_EXTENSION))
				return TestUIImages.INSTANCE.getImage(TestUIImages.IMG_EXECUTION);
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) 
	{
		if (element instanceof IFile) 
		{
			IFile file = (IFile) element;
			return file.getParent();
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
	public String getText(Object element) 
	{
		if (element instanceof IFile) 
		{
			IFile file = (IFile) element;
			String name = file.getName();
			String ext = file.getFileExtension();
			if (ext.equals(ICommonConstants.EXECUTION_FILE_EXTENSION)) {
				//return name.substring(0, name.length() - ext.length() - 1);
				String fileName = name.substring(0, name.length() - ext.length() - 1);
				Object[] tab = getChildren(file);  
				if ( tab[0]!=null && tab[0] instanceof TPFExecutionResult) {
					TPFExecutionResult er = (TPFExecutionResult) tab[0];
					if (er.getExecutionHistory().getExecutionEvents()!=null && er.getExecutionHistory().getExecutionEvents().size()>0) {
						long timeStamp = ((TPFExecutionEvent)er.getExecutionHistory().getExecutionEvents().get(0)).getTimestamp();
						String dateStamp = EventUtil.getTime(timeStamp);
						fileName += " ["+dateStamp+"]";  //$NON-NLS-1$//$NON-NLS-2$
					}
				}
				return fileName;
			}
				
		}
		return ""; //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) 
	{
		if (element instanceof IFile) 
		{
			IFile executionfile = (IFile) element;
			String extension = executionfile.getFileExtension();
			if (extension.equals(ICommonConstants.EXECUTION_FILE_EXTENSION) )
			{
				EObject[] eobjects = EMFUtil.load(null, executionfile);
				for ( int i = 0; i < eobjects.length; i++ )
				{
					if ( eobjects[i] instanceof TPFExecutionResult )
					   return true;
				}
			}
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(org.eclipse.core.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
	 */
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data) throws CoreException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener(ILabelProviderListener listener) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
	public boolean isLabelProperty(Object element, String property) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener(ILabelProviderListener listener) {
		// TODO Auto-generated method stub

	}

	/** This method returns the verdict of the passed element, in this case there is no status
	 * because this is not an execution node.
	 * @param element the element in the Test Navigator
	 * @return  IHyadesTestNavigatorProvider.VERDICT_NONE
	 */
	public int getVerdict(Object element)
	{
		//TODO: KT: see if I return the worst severity of children?
		return IHyadesTestNavigatorProvider.VERDICT_NONE;
	}

}
