/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: HotSpot.java,v 1.3 2005/02/16 22:24:06 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.uml2sd.ui.core.internal;

import org.eclipse.hyades.uml2sd.ui.core.BasicExecutionOccurrence;
import org.eclipse.hyades.uml2sd.ui.core.Frame;
import org.eclipse.hyades.uml2sd.ui.core.GraphNode;
import org.eclipse.hyades.uml2sd.ui.drawings.IGC;
import org.eclipse.hyades.uml2sd.ui.drawings.IImage;
import org.eclipse.hyades.uml2sd.ui.drawings.ISDPreferences;

/**
 * BasicExecutionOccurrence is the UML2 execution occurrence graphical representation.
 * It is attached to one Lifeline, the event occurrence "duration" along 
 * the lifeline is defined by two event occurrences 
 * 
 * @see org.eclipse.hyades.uml2sd.ui.core.Lifeline Lifeline for more event occurence details
 * @author sveyrier
 *
 */
public class HotSpot extends GraphNode {
	
	protected BasicExecutionOccurrence execOcc	= null;
	protected int 		occurrence = 0;
	protected IImage	image = null;
	
	/**
	 * The grahNode ID constant
	 */
	public static final String GLYPH = "Glyph"; //$NON-NLS-1$
	
	public HotSpot()
	{
		prefId = ISDPreferences.PREF_EXEC;
	}
	
	public void setImage(IImage img)
	{
		image=img;
	}

	public int getX()
	{
		if (execOcc!=null)
			return execOcc.getX()-3;
		else return 0;
		
	}

	public int getY()
	{
		if (execOcc!=null)
			return execOcc.getY();
		else return 0;
	}

	public int getWidth()
	{
		if (execOcc!=null)
			return execOcc.getWidth()+7;
		else return 0;
	}

	
	public int getHeight()
	{
		if (execOcc!=null)
			return execOcc.getWidth()+10;
		else return 0;
	}	
	
	/**
	 * Set the lifeline on which the execution occurrence appears. 
	 * @param theLifeline - the parent lifeline
	 */
	public void setExecution (BasicExecutionOccurrence occ)
	{
		execOcc = occ;
		execOcc.addNode(this);
	}
	
	/**
	 * Get the lifeline on which the execution occurrence appears. 
	 * @return - the parent lifeline
	 */
	public BasicExecutionOccurrence getExecOcc ()
	{
		return execOcc;
	}
	
	public int getOccurrence ()
	{
		return occurrence;
	}
	
	public void setOccurrence (int occ)
	{
		occurrence = occ;
	}

	public void draw (IGC context)
	{
		
//		The execution occurrence is selected
		//if the owning lifeline is selected
		if (isSelected()||
				(execOcc!=null&&execOcc.isSelected())||
				(execOcc!=null&&execOcc.getLifeline()!=null&&
				execOcc.getLifeline().isSelected()))
		{
			context.setBackground(Frame.getUserPref().getBackGroundColorSelection());
			context.setForeground(Frame.getUserPref().getForeGroundColorSelection());
		}
		else 
		{
			context.setBackground(Frame.getUserPref().getBackGroundColor(ISDPreferences.PREF_EXEC));
			context.setForeground(Frame.getUserPref().getForeGroundColor(ISDPreferences.PREF_EXEC));
		}
		context.drawImage(image,getX() ,getY(), getWidth(), getHeight());
	}
	
	public String getArrayId()
	{
	 	return GLYPH;
	}
	
	public boolean isVisible(int x, int y, int width, int height)
	{
		return true;
	}
	
	
	public boolean contains(int _x, int _y) 
	{
		int x = getX();
		int y = getY();
		int width = getWidth();
		int height = getHeight();
		
		if (Frame.contains(x, y, width, height,_x, _y))
		{
			return true;
		}
		return false;
	}
}
